import pandas as pd

query_string = """
  query LiquidityPools($limit: Int!, $orderBy: String!, $orderDir: String!, $protocol: String, $symbol: String) {
    liquidityPools(
      limit: $limit
      orderBy: $orderBy
      orderDir: $orderDir
      protocol: $protocol
      symbol: $symbol
    ) {
      address
      swapFee
      protocol
      reserveUsd
      volumeUsd
      vlmLiqRatio
      tokens {
        symbol
        balance
        weight
      }
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['liquidityPools'])


import sys
sys.path.append('./lib')

import requests
import json
import pandas as pd
from parsecfi.lib import endpoints

BASE_URL = "https://api.parsec.finance/api/v2"

class Parsec:
	def __init__(self, ak):
		self.api_key = ak
		self.endpoints = endpoints.load()

	def api_call(self, endpoint, variables):
		json_data = endpoint.query(variables)
		r = requests.post(BASE_URL, json=json_data, headers={'api_key': self.api_key})
		if r.status_code == 200:
			text = json.loads(r.text)
			if 'data' in text:
				return endpoint.parser(text['data'])
			else:
				return text
		else:
			return f"Error: {r.status_code}"

	def __getattr__(self, name):
		if name in self.endpoints:
			return lambda variables: self.api_call(endpoint=self.endpoints[name], variables=variables)
		else:
			raise AttributeError(f"'Parsec' object has no attribute '{name}'")



from parsecfi.lib.endpoints_list import (
	lending_market, 
	contract_logs, 
	token_holders, 
	token_acc, 
	trending_contracts,
	transfers,
	address,
	transactions
)

def load():
	return {
		'lending_market': lending_market,
		'contract_logs': contract_logs,
		'token_holders': token_holders,
		'token_acc': token_acc,
		'trending_contracts': trending_contracts,
		'transfers': transfers,
		'address': address,
		'transactions': transactions
	}


import pandas as pd

query_string = """
  query TradeMetricsBins($chains: [String], $limit: Int!, $offset: Int, $venues: [String], $pairs: [String]) {
	tradeMetricsBins(chains: $chains, limit: $limit, interval: $interval, offset: $offset, venues: $venues, pairs: $pairs) {
	  uniqueTakers
	  volumeUsd
	  nTrades
	  startTs
	  endTs
	}
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['tradeMetricsBins'])




import pandas as pd

query_string = """
  query NftCollection($address: String!) {
    nftCollection(address: $address) {
      name
      floor
      address
      chain
      holdersCount
      floor1dDelta
      floor7dDelta
      floor30dDelta
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['token']['topHolders'])


import pandas as pd

query_string = """
  query NftTrades(
    $limit: Int!, 
    trader: $trader, 
    traits: [[String]], 
    since: Int,
    before: Int,
    collectionAddress: String,
    tokenId: String
    exchange: String
  ) {
    nftTrades(limit: $limit) {
      chain
      price
      side
      usdValue
      thumbnailUrl
      tokenId
      taker
      maker
      timestamp
      usdValue
      collectionAddress
      collectionName
      exchange
      tx
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['nftTrades'])


import pandas as pd

query_string = """
  query Address($name: String, $address: String) {
    address(name: $name, address: $address) {
      name
      address
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return json_data['address']


import pandas as pd

query_string = """
	query LendingMarket($symbol: String!, $protocol: String!) {
		lendingMarket(symbol: $symbol, protocol: $protocol) {
			poolId
			poolName
			protocol
			supplyRate
			borrowRate
			collatRatio
			history(interval: "1d", limit: 30) {
				borrowRate
				supplyRate
				totalSupplied
				totalBorrows
				borrowRewards {
				rateBase
				rewardToken
			}
		}
	}
}
"""

def query(variables):
	return {
		'query': query_string,
		'variables': variables
	}

def parser(json_data):
	return pd.DataFrame(json_data['lending_market'])


import json
import pandas as pd

query_string = """
	query ContractLogs(
		$address: String!, 
		$chain: String!, 
		$log: String!,
		$since: Int,
		$limit: Int,
		$filters: [LogFilter]
	) {
	  contract(address: $address, chain: $chain) {
	  	logs(limit: $limit, log: $log, since: $since, filters: $filters) {
	      timestamp
	      txId
	      data
	    }
	  }
	}
"""

def query(variables):
	return {
		'query': query_string,
		'variables': variables
	}

def parser(json_data):
	data = json_data['contract']['logs']

	for item in data:
		d = json.loads(item['data'])
		for key, val in d.items():
			item[key] = val
		del item['data']

	return pd.DataFrame(data)


import pandas as pd

query_string = """
  query Trades(
    $since: Int,
    $before: Int,
    $from: String,
    $taker: String,
    $maker: String,
    $pair: String,
    $side: String,
    $sort: [String],
    $tx: String,
    $venue: String,
    $limit: Int!
  ) {
    trades(limit: $limit) {
      venue
      baseAmount
      from
      pair
      price
      quoteAmount
      side
      timestamp
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['trades'])


import pandas as pd

query_string = """
  query LpPositions($poolAddress: String!, $inRangeOnly: Boolean!, $exchange: String!, $limit: Int!) {
    liquidityPool(poolAddress: $poolAddress, exchange: $exchange) {
      positions(limit: $limit, inRangeOnly: $inRangeOnly) {
        deltaQuote
        feeBps
        nav
        pair
        positionId
        price
        provider
        tickLowerPrice
        tickUpperPrice
        addressLabel {
          src
          label
          address
        }
      }
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['liquidityPool']['positions'])


import pandas as pd

query_string = """
  query GetCandles($interval: String!, $limit: Int!, $pair: String!, $venue: String!) {
    candles(interval: $interval, limit: $limit, pair: $pair, venue: $venue) {
      data
      derived
      exchange
      interval
      pair
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['candles'])


import json
import pandas as pd

query_string = """
	query Transfers(
		$address: String, 
		$to: String,
		$chain: String,
		$from: String,
		$toOrFrom: String,
		$toOrFroms: [String],
		$tx: String,
		$block: Int,
		$limit: Int,
		$offset: Int
	) {
		transfers(
			address: $address,
			chain: $chain, 
			to: $to,
			from: $from, 
			toOrFrom: $toOrFrom,
			toOrFroms: $toOrFroms,
			tx: $tx,
			block: $block,
			limit: $limit,
			offset: $offset
		) {
			to
		    from
		    value
		    timestamp
		    tx
		    address
		    decimals
		    usd_price
		    usd_value
		    symbol
		    img_src
		    chain
		    is_nft
		}
	}
"""

def query(variables):
	return {
		'query': query_string,
		'variables': variables
	}

def parser(json_data):
	return pd.DataFrame(json_data['transfers'])


import pandas as pd

query_string = """
  query TrendingContracts($chain: String!, $sortBy: String!, $limit: Int!, $lookback: String!) {
    trendingContracts(
      chain: $chain
      sortBy: $sortBy
      limit: $limit
      lookback: $lookback
    ) {
      address
      chain
      usage {
        ethSent
        gas
        gasPrev
        nTxs
        senders
        sendersChange
        sendersPrev
      }
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  data = json_data['trendingContracts']

  for item in data:
    for key, val in item['usage'].items():
      item[key] = val
    del item['usage']

  return pd.DataFrame(data)


import pandas as pd

query_string = """
  query TokenHolders($address: String!, $chain: String!, $limit: Int!) {
    token(address: $address, chain: $chain) {
      topHolders(limit: $limit) {
        balance
        address
        addressLabel {
          label
        }
      }
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['token']['topHolders'])



from setuptools import setup, find_packages

setup(
    name='parsecfi',
    version='0.1.2',
    description='Python SDK for the Parsec API',
    author='PARSEC FINANCE INC.',
    packages=[
        'parsecfi',
        'parsecfi.lib',
        'parsecfi.lib.endpoints_list'
    ],
    license='MIT',
    install_requires=[
        'requests',
        'pandas'
    ]
)


import json
import pandas as pd

query_string = """
  query ApiTxs($addresses: [String!]!, $limit: Int!, $chains: [String], $offset: Int, $call: String, $to: String, $from: String, $since: Int, $before: Int) {
    txs(
      addresses: $addresses
      limit: $limit
      chains: $chains
      offset: $offset
      functionCall: $call
      to: $to
      from: $from
      since: $since
      before: $before
    ) {
      chain
      block
      hash
      to
      from
      gasPrice
      gasUsed
      input
      call
      status
      index
      value
      timestamp
      meta
      contract_creation
      marketData {
        type
        data
        __typename
      }
      inputDecoded {
        function
        args
        __typename
      }
      message
      __typename
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['txs'])


import pandas as pd

query_string = """
  query TokenDeltas($address: String, $limit: Integer!, $lookback: String!, $chain: String!) {
    token(address: $address, chain: $chain) {
      balanceChanges(limit: $limit, lookback: $lookback) {
        delta
        balanceStart
        balanceEnd
        bought
        sold
        chain
        address
        addressLabel {
          label
        }
      }
    }
  }
"""

def query(variables):
  return {
    'query': query_string,
    'variables': variables
  }

def parser(json_data):
  return pd.DataFrame(json_data['token']['balanceChanges'])

def help():
  f"Find the largest token deltas over an interval"